// Copyright (C) 2023 OWO Games - All Rights Reserved

#include "UOWOController.h"

uniquePtr<OWO> UOWOController::owo = nullptr;

void UOWOController::Initialize()
{
    if (owo == nullptr)
        owo = OWO::Create<UDPNetwork>();
}

void UOWOController::BeginDestroy() {

    if (owo != nullptr) {
        owo->Stop();
        owo->Disconnect();
    }

    Super::BeginDestroy();
}

EOWOConnectionState ConvertStatus(ConnectionState state) {
    switch (state) {
    case ConnectionState::Disconnected:
        return EOWOConnectionState::Disconnected;
    case ConnectionState::Connecting:
        return EOWOConnectionState::Connecting;
    case ConnectionState::Connected:
        return EOWOConnectionState::Connected;
    }

    return EOWOConnectionState::Disconnected;
}

void UOWOController::MultiConnect(TArray<FString> ips)
{
    Initialize();

    owoVector<owoString> servers;

    for (int i = 0; i < ips.Num(); i++) {
        owoString IP = owoString(TCHAR_TO_UTF8(*ips[i]));
        if (!IP.empty())
            servers.push_back(IP);
    }

    owo->Connect(servers);
}

void UOWOController::Connect(const FString& InAddress)
{
    Initialize();

    std::string IP = std::string(TCHAR_TO_UTF8(*InAddress));

    if (IP.empty()) return;
    owo->Connect({ IP });
}

void UOWOController::Configure(std::vector<std::string> parameters)
{
    sharedPtr<GameAuth> authParsed = GameAuth::Parse(parameters[2], parameters[0]);
    owo->Configure(authParsed);
}

void UOWOController::Configure(FString gameAuth)
{
    Initialize();

    std::string auth = std::string(TCHAR_TO_UTF8(*gameAuth));
    std::vector<std::string> parameters = String::Split(auth, '*');
#if WITH_EDITOR
    try {
        Configure(parameters);
    }
    catch (WrongGameAuthFormatException e) {
        std::string auxError = e.GetCode();
        FString error = auxError.c_str();
        UE_LOG(LogTemp, Error, TEXT("%s"), *error);
        GEngine->AddOnScreenDebugMessage(-1, 5.f, FColor::Red, FString::Printf(TEXT("%s"), *error));
    }
#else
    Configure(parameters);
#endif
}

FString UOWOController::CreateGameAuth(FString gameId, TArray<FString> bakedSensations) {
    Initialize();

    std::string gameIdString = std::string(TCHAR_TO_UTF8(*gameId));
    std::vector<owoString> sensations;

    if (gameIdString.size() < 1) {
        gameIdString = "0";
    }

    for (int i = 0; i < bakedSensations.Num(); i++) {
        if (!bakedSensations[i].IsEmpty()) {
                sensations.push_back(std::string(TCHAR_TO_UTF8(*bakedSensations[i])));
        }
    }

    GameAuth auth = GameAuth(sensations, gameIdString);
    std::string result = auth.id + "*AUTH*" + auth.ToString();
    return result.c_str();
}

void UOWOController::Autoconnect()
{
    Initialize();

    if (owo->State() == ConnectionState::Connecting) return;

    owo->AutoConnect();
}

void UOWOController::Disconnect()
{
    Initialize();

    owo->Disconnect();
}

void UOWOController::SendSensation(FString sensation, FString muscleFString, int priority)
{
    Initialize();

    std::string sensationString = std::string(TCHAR_TO_UTF8(*sensation));

    uniquePtr<Sensation> sensationParsed = SensationsParser::Parse(sensationString);
    if (!muscleFString.IsEmpty()) {
        std::string muscles = std::string(TCHAR_TO_UTF8(*muscleFString));
        sensationParsed = sensationParsed->WithMuscles(MusclesParser::Parse(muscles));
    }

    sensationParsed->SetPriority(priority);
    owo->Send(movePtr(sensationParsed));

}

void UOWOController::Scan(float timeInSeconds) {
    Initialize();

    owo->Scan(timeInSeconds * 1000);
}

EOWOConnectionState UOWOController::UpdateStatus(float timeInSeconds)
{
    Initialize();
    if (owo->State() == ConnectionState::Disconnected) return ConnectionStatus();

    owo->UpdateStatus(timeInSeconds * 1000);

    return ConnectionStatus();
}

TArray<FString> UOWOController::GetAvailableServers() {
    Initialize();

    owoVector<owoString> servers = owo->DiscoveredApps();

    TArray<FString> result;

    for (int i = 0; i < servers.size(); i++) {

        result.Add(servers[i].c_str());
    }

    return result;
}

void UOWOController::StopCurrentSensation()
{
    Initialize();

    owo->Stop();
}

EOWOConnectionState UOWOController::ConnectionStatus() {
    Initialize();
    return ConvertStatus(owo->State());
}

